package MT::Plugin::RebuildAt1stView;
use strict;
use MT;
use MT::Template;
use MT::Task;
use MT::Permission;
use MT::PluginData;
use RebuildAt1stView::BackgroundRebuild;
require MT::Plugin;

our $VERSION = '0.7';
our $SCHEMA_VERSION = '0.4';

use base qw( MT::Plugin );

###################################### Init Plugin #####################################

my $plugin = MT::Plugin::RebuildAt1stView->new({
    id => 'RebuildAt1stView',
    key => 'rebuild-at-1st-view',
    description => '<MT_TRANS phrase=\'_PLUGIN_DESCRIPTION\'>',
    name => 'RebuildAt1stView',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    'version' => $VERSION,
    l10n_class => 'RebuildAt1stView::L10N',
    blog_config_template => 'config.tmpl',
    system_config_template => 'sysconfig.tmpl',
    schema_version => $SCHEMA_VERSION,
    settings => new MT::PluginSettings([
        ['RebuildAt1stView', { Default => 0 }],
        ['RebuildAt1stViewTask', { Default => 0 }],
        ['SleepTime', { Default => 1 }],
        ['WaitAtRedirect', { Default => 0.5 }],
        ['CMSSleepTime', { Default => 1 }],
        ['EntriesPerRebuild', { Default => 10 }],
        ['BackgroundRebuilder', { Default => 0 }],
    ]),
});

MT->add_plugin($plugin);

sub instance { $plugin }

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        object_types => {
            'template' => {
                'rebuild_at' => 'integer',
            },
            'backgroundrebuild' =>  'RebuildAt1stView::BackgroundRebuild',
        },
        callbacks => {
            'build_file_filter'
                => \&_build_file_filter,
            'MT::App::CMS::template_param.edit_template'
                => \&_rebuild_flag,
            'MT::App::CMS::template_output.list_blog'
                => \&_RebuildAt1stViewOption,
            'MT::App::CMS::template_source.list_blog'
                => \&_RebuildAt1stViewInclude,
            'MT::App::CMS::template_output.list_template'
                => \&_RebuildAt1stViewIncludeOptionTmplates,
            'MT::App::CMS::template_source.list_template'
                => \&_RebuildAt1stViewIncludeTmplates,
            'MT::App::CMS::template_param'
                => \&_add_rebuild_status,
            'MT::App::CMS::template_source.edit_entry'
                => {
                    handler => \&_rebuild_entry,
                    priority => 9,
                },
            'MT::App::CMS::template_source.rebuilt'
                => {
                    handler => \&_finish_and_rebuild,
                    priority => 9,
                },
        },
        tasks => {
            'RebuildAt1stViewTask' => {
                label     => $plugin->translate('RebuildAt1stView Task'),
                frequency => 300,
                code      => \&_build_file,
            },
        },
        applications => {
            cms => {
                list_actions => {
                    blog => {
                        rebuildat1stview_on => {
                            label => $plugin->translate('RebuildAt1stView Active'),
                            code => \&_active,
                            permissions => 'manage_plugins',
                        },
                        rebuildat1stview_off => {
                            label => $plugin->translate('RebuildAt1stView Deactive'),
                            code => \&_deactive,
                            permissions => 'manage_plugins',
                        },
                    },
                    template => {
                        rebuild_at_realtime => {
                            label => $plugin->translate('Rebuild at realtime'),
                            code => \&_set_realtime,
                            permissions => 'can_edit_templates',
                        },
                        rebuild_at_1stview => {
                            label => $plugin->translate('Rebuild at 1st view'),
                            code => \&_set_1stview,
                            permissions => 'can_edit_templates',
                        },
                    },
                },
                menus => {
                    'tools:rebuild_at_cms' => {
                        label => $plugin->translate('Rebuild Static Files'),
                        mode => 'rebuild_at_cms',
                        order => 7000,
                        permission => 'view_blog_log',
                    },
                },
                methods => {
                    rebuild_at_cms =>
                        'MT::Plugin::RebuildAt1stView::_rebuild_at_cms',
                },
            },
        },
   });
}

###################################### Plugin Action ####################################

sub _active {
    my $app = shift;
    my @id = $app->param('id') or return $app->errtrans('Invalid request.');
    my $user = $app->user;
    my $perms = MT::Permission->load({ blog_id => 0,
                                       author_id => $user->id });
    unless ($perms) {
        return $app->errtrans('Permission Denied.');
    }
    my $permissions = $perms->permissions;
    unless ($permissions =~ /manage_plugins/) {
        return $app->errtrans('Permission Denied.');
    }
    foreach my $id (@id) {
        next unless $id;
        my $plugin_data = MT::PluginData->get_by_key({ plugin => 'rebuild-at-1st-view',
                                                       key => 'configuration:blog:' . $id });
        my $datas = $plugin_data->data;
        $datas->{RebuildAt1stView} = 1;
        $plugin_data->data($datas); 
        $plugin_data->save
            or return $app->trans_error('Error saving plugindata: [_1]', $plugin_data->errstr);
    }
    $app->call_return;
}

sub _deactive {
    my $app = shift;
    my @id = $app->param('id') or return $app->errtrans('Invalid request.');
    my $user = $app->user;
    my $perms = MT::Permission->load({ blog_id => 0,
                                       author_id => $user->id });
    unless ($perms) {
        return $app->errtrans('Permission Denied.');
    }
    my $permissions = $perms->permissions;
    unless ($permissions =~ /manage_plugins/) {
        return $app->errtrans('Permission Denied.');
    }
    foreach my $id (@id) {
        next unless $id;
        my $plugin_data = MT::PluginData->get_by_key({ plugin => 'rebuild-at-1st-view',
                                                       key => 'configuration:blog:' . $id });
        my $datas = $plugin_data->data;
        $datas->{RebuildAt1stView} = 0;
        $plugin_data->data($datas); 
        $plugin_data->save
            or return $app->trans_error('Error saving plugindata: [_1]', $plugin_data->errstr);
    }
    $app->call_return;
}

sub _set_realtime {
    my $app = shift;
    my @id = $app->param('id') or return $app->errtrans('Invalid request.');
    my $user = $app->user;
    foreach my $id (@id) {
        next unless $id;
        require MT::Template;
        my $temp = MT::Template->load($id);
        next unless $temp;
        my $perms = $user->permissions($temp->blog_id);
        next unless $perms && $perms->can_edit_templates;
        $temp->rebuild_at(1);
        $temp->save
            or return $app->trans_error('Error saving template: [_1]', $temp->errstr);
    }
    $app->call_return;
}

sub _set_1stview {
    my $app = shift;
    my @id = $app->param('id') or return $app->errtrans('Invalid request.');
    my $user = $app->user;
    foreach my $id (@id) {
        next unless $id;
        require MT::Template;
        my $temp = MT::Template->load($id);
        next unless $temp;
        my $perms = $user->permissions($temp->blog_id);
        next unless $perms && $perms->can_edit_templates;
        $temp->rebuild_at(0);
        $temp->save
            or return $app->trans_error('Error saving template: [_1]', $temp->errstr);
    }
    $app->call_return;
}

######################################### Do Menu ########################################

sub _rebuild_at_cms {
    my $app = shift;
    if ( $plugin->get_config_value('RebuildAt1stViewTask') == 1) {
        #$plugin->set_config_value('RebuildAt1stViewTask', 0);
        return $app->trans_error('Other Rebuilding Process is Running.');
    }        
    $plugin->set_config_value('RebuildAt1stViewTask', 1);
    my $base = $app->base;
    my $q = $app->param;
    my $blog_id = $q->param('blog_id');
    my $user = $app->user;
    if ( $blog_id ) {
        my $perms = $user->permissions($blog_id);
        unless ($perms->rebuild) {
            unless ( $user->is_superuser ) {
                return $app->trans_error('Permission Denied.');
            }
        }
        my $get_from = 'blog:'.$blog_id;
        my $plugin_active = $plugin->get_config_value('RebuildAt1stView', $get_from);
        unless ( $plugin_active ) {
            return $app->trans_error('RebuildAt1stView is deactive.');
        }
    } else {
        unless ( $user->is_superuser ) {
            return $app->trans_error('Permission Denied.');
        }
    }
    my $start_val = $q->param('start_val');
    my $limit = $plugin->get_config_value('EntriesPerRebuild');
    my $sleepTime = $plugin->get_config_value('CMSSleepTime');
    $app->{plugin_template_path} = File::Spec->catdir($plugin->path,'tmpl');
    my $tmpl = 'static_rebuild.tmpl';
    my $js_wait = $plugin->get_config_value('WaitAtRedirect');
    $js_wait = $js_wait * 1000;
    my %param;
    $param{'page_title'} = $plugin->translate('Rebuilding Static Files');
    $param{'rebuild_report'} = 1;
    $param{'page_msg'} = $plugin->translate('Rebuilding Static Files...');
    unless ($start_val) {
        my $return_url = $app->uri(mode => 'rebuild_at_cms', args => { 'start_val' => 1, 'blog_id' => $blog_id } );
        my $js = <<"JS";
    var counter = 0;
    var eid;
    function func() {
        if (counter != 0) {
            clearTimeout(eid);
            counter = 0;
        } else {
            counter = 1;
            eid = setTimeout("func(window.location.href = '$base$return_url')", $js_wait);
        }
    }
    func();
JS
        $param{'location_redirect'} = $js;
        $plugin->set_config_value('RebuildAt1stViewTask', 0);
        return $app->build_page($tmpl, \%param);
    }
    $plugin->set_config_value('RebuildAt1stViewTask', 1);
    if ( $start_val == 1 ) {
        $start_val = 0;
    }
    my $terms = {};
    if ( $blog_id ) {
        $terms->{'blog_id'} = $blog_id;
    }
    my $params = {}; my $count_params = {}; 
    $params->{'sort'} = 'id';
    $count_params->{'sort'} = 'id';
    $params->{'direction'} = 'ascend';
    $count_params->{'direction'} = 'descend';
    $params->{'limit'} = $limit;
    $params->{'start_val'} = $start_val;
    $count_params->{'start_val'} = $start_val;
    require MT::FileInfo;
    my @fileinfos = MT::FileInfo->load( $terms, $params );
    my @rebuild_items;
    if (@fileinfos) {
        my $progress; my $total;
        if ($start_val) {
            $progress= MT::FileInfo->count( $terms, $count_params );
            $total= MT::FileInfo->count( undef );
            my $percent = int ( ( $progress / $total ) * 100 );
            $param{'page_bar_length'} = int ( $percent * 9.2 );
        }
        for my $fi (@fileinfos) {
            my $fid = $fi->id;
            my @fis = MT::FileInfo->load({
                                    url => $fi->url,
                                  }, {
                                    sort => 'id',
                                    direction => 'ascend',
                                  });
            my $i = 1;
            my $tmp_fi;
            if (@fis > 1) {
                for my $finfo (@fis) {
                    if ($i == 1) {
                        $tmp_fi = $finfo;
                    } else {
                        $finfo->remove
                            or return $app->trans_error('Error removing fileinfo: [_1]', $finfo->errstr);
                    }
                    $i++;
                }
                if ( $fid == $tmp_fi->id ) {
                    push (@rebuild_items, $tmp_fi);
                }
            } else {
                push (@rebuild_items, $fi);
            }
        }
        if ( @rebuild_items ) {
            require MT::WeblogPublisher;
            my $pub = MT::WeblogPublisher->new;
            my $last_id;
            for my $fi ( @rebuild_items ) {
                my $file_path = $fi->file_path;
                my $get_from = 'blog:'.$fi->blog_id;
                my $plugin_active = $plugin->get_config_value('RebuildAt1stView', $get_from);
                next unless $plugin_active;
                $last_id = $fi->id;
                unless (-f $file_path) {
                    my $template = MT::Template->load( { id => $fi->template_id } );
                    unless (defined $template) {
                        $fi->remove
                            or return $app->trans_error('Error removing fileinfo: [_1]', $fi->errstr);
                        next;
                    } else {
                        next if $template->build_dynamic;
                    }
                    if ($sleepTime) {
                        sleep $sleepTime;
                    }
                    my $res = $pub->rebuild_from_fileinfo($fi);
                    unless (-f $file_path) {
                        my $at = $fi->archive_type;
                        my $entry;
                        if ( $fi->entry_id ) {
                            $entry = MT::Entry->load( { id => $fi->entry_id } );
                            if ( defined $entry ) {
                                if ( $entry-> status == 2 ) {
                                    $pub->rebuild_entry( {
                                        Entry  => $entry,
                                        Blog   => MT::Blog->load( { id => $entry->blog_id } ),
                                        BuildDependencies => 0,
                                    });
                                } else {
                                    next;
                                }
                            }
                        } else {
                            my ( $start, $end );
                            my $blog = MT::Blog->load( { id => $fi->blog_id } );
                            if ($fi->startdate) {
                                my $archiver = $pub->archiver( $at );
                                if ( my $range = $archiver->date_range ) {
                                    ( $start, $end ) = $range->( $fi->startdate );
                                    $entry = MT::Entry->load( { authored_on => [ $start, $end ] },
                                                              { range_incl => { authored_on => 1 }, limit => 1 } );
                                }
                            }
                            my $cat;
                            if ( $fi->category_id ) {
                                $cat = MT::Category->load( { id => $fi->category_id } );
                            }
                            my $map = MT::TemplateMap->load( { id => $fi->templatemap_id } );
                            my $file = $pub->archive_file_for( $entry, $blog, $at, $cat, $map );
                            unless ( defined($file) ) {
                                next;
                            }
                            $map->{__saved_output_file} = $file;
                            my $ctx = MT::Template::Context->new;
                            $ctx->{current_archive_type} = $at;
                            if ( $start && $end ) {
                                $ctx->{current_timestamp} = $start;
                                $ctx->{current_timestamp_end} = $end;
                            }
                            my $arch_root = ( $at eq 'Page' ) ? $blog->site_path : $blog->archive_path;
                            my %cond;
                            $pub->rebuild_file( $blog, $arch_root, $map, $at, $ctx, \%cond, 1, FileInfo => $fi );
                        }
                        unless (-f $file_path) {
                            require MT::Log;
                            my $log = MT::Log->new;
                            $log->message($plugin->translate('An error occurred while rebuild file [_1].', $fi->url));
                            $log->save
                                or die $log->errstr;
                        }
                    }
                }
            }
            my $return_url = $app->uri(mode => 'rebuild_at_cms', args => { 'start_val' => $last_id, 'blog_id' => $blog_id } );
            my $js = <<"JS";
    var counter = 0;
    var eid;
    function func() {
        if (counter != 0) {
            clearTimeout(eid);
            counter = 0;
        } else {
            counter = 1;
            eid = setTimeout("func(window.location.href = '$base$return_url')", $js_wait);
        }
    }
    func();
JS
            $param{'location_redirect'} = $js;
            $plugin->set_config_value('RebuildAt1stViewTask', 0);
            return $app->build_page($tmpl, \%param);
        }
    }
    $param{'page_title'} = $plugin->translate('Done.');
    $param{'rebuild_report'} = 0;
    $param{'page_msg'} = $plugin->translate('All of your files have been published.');
    $plugin->set_config_value('RebuildAt1stViewTask', 0);
    return $app->build_page($tmpl, \%param);
}

####################################### Plugin Task ######################################

sub _build_file {
    $plugin->set_config_value('RebuildAt1stViewTask', 1);
    my $app = MT->new;
    require MT::FileInfo;
    my $fi;
    my $iter = MT::FileInfo->count_group_by( undef,
        { group => ['url', 'file_path']} );
    my @redundancy = ();
    while (my ($count, $url, $file_path) = $iter->()) {
        if ($count != 1) {
            push (@redundancy, $url);
        }
    }
    if (@redundancy) {
        for my $url (@redundancy) {
            my @fis = MT::FileInfo->load({
                                            url => $url,
                                          }, {
                                            sort => 'id',
                                            direction => 'descend',
                                          });
            my $i = 1;
            if (@fis > 1) {
                for my $finfo (@fis) {
                    unless ($i == 1) {
                        $finfo->remove
                            or return $app->trans_error('Error removing fileinfo: [_1]', $finfo->errstr);
                    }
                    $i++;
                }
            }
        }
    }
    my $fileinfos = MT::FileInfo->load_iter( undef );
    my $sleepTime = $plugin->get_config_value('SleepTime');
    while (my $fi = $fileinfos->()) {
        my $blog_id = $fi->blog_id;
        my $get_from = 'blog:'.$blog_id;
        my $plugin_active = $plugin->get_config_value('RebuildAt1stView', $get_from);
        next unless $plugin_active;
        my $file_path = $fi->file_path;
        unless (-f $file_path) {
            my $template = MT::Template->load( { id => $fi->template_id } );
            unless (defined $template) {
                $fi->remove
                    or return $app->trans_error('Error removing fileinfo: [_1]', $fi->errstr);
                next;
            } else {
                next if $template->build_dynamic;
            }
            require MT::WeblogPublisher;
            my $pub = MT::WeblogPublisher->new;
            my $res = $pub->rebuild_from_fileinfo($fi);
            unless (-f $file_path) {
                my $at = $fi->archive_type;
                my $entry;
                if ( $fi->entry_id ) {
                    $entry = MT::Entry->load( { id => $fi->entry_id } );
                    if ( defined $entry ) {
                        if ( $entry-> status == 2 ) {
                            $pub->rebuild_entry( {
                                Entry  => $entry,
                                Blog   => MT::Blog->load( { id => $entry->blog_id } ),
                                BuildDependencies => 0,
                            });
                        } else {
                            next;
                        }
                    }
                } else {
                    my ( $start, $end );
                    my $blog = MT::Blog->load( { id => $fi->blog_id } );
                    if ($fi->startdate) {
                        my $archiver = $pub->archiver( $at );
                        if ( my $range = $archiver->date_range ) {
                            ( $start, $end ) = $range->( $fi->startdate );
                            $entry = MT::Entry->load( { authored_on => [ $start, $end ] },
                                                      { range_incl => { authored_on => 1 }, limit => 1 } );
                        }
                    }
                    my $cat;
                    if ( $fi->category_id ) {
                        $cat = MT::Category->load( { id => $fi->category_id } );
                    }
                    my $map = MT::TemplateMap->load( { id => $fi->templatemap_id } );
                    my $file = $pub->archive_file_for( $entry, $blog, $at, $cat, $map );
                    unless ( defined($file) ) {
                        next;
                    }
                    $map->{__saved_output_file} = $file;
                    my $ctx = MT::Template::Context->new;
                    $ctx->{current_archive_type} = $at;
                    if ( $start && $end ) {
                        $ctx->{current_timestamp} = $start;
                        $ctx->{current_timestamp_end} = $end;
                    }
                    my $arch_root = ( $at eq 'Page' ) ? $blog->site_path : $blog->archive_path;
                    my %cond;
                    $pub->rebuild_file( $blog, $arch_root, $map, $at, $ctx, \%cond, 1, FileInfo => $fi );
                }
                unless (-f $file_path) {
                    require MT::Log;
                    my $log = MT::Log->new;
                    $log->message($plugin->translate('An error occurred while rebuild file [_1].', $fi->url));
                    $log->save
                        or die $log->errstr;
                }
            }
            if ( $sleepTime ) {
                sleep $sleepTime;
            }
        }
    }
    $plugin->set_config_value('RebuildAt1stViewTask', 0);
}

####################################### Filter Callback ######################################

sub _build_file_filter {
    my ($eh, %args) = @_;
    my $blog = $args{'Blog'};
    my $blog_id = $blog->id;
    my $get_from = 'blog:'.$blog_id;
    my $is_task = $plugin->get_config_value('RebuildAt1stViewTask');
    if ($is_task) {
        return 1;
    }
    my $plugin_active = $plugin->get_config_value('RebuildAt1stView', $get_from);
    if ($plugin_active) {
        my $template = $args{'Template'};
        if ($template->rebuild_at) {
            return 1;
        }
        my $redirect_url = $ENV{'REDIRECT_URL'};
        my $script_path = $ENV{'SCRIPT_NAME'};
        if ( ($redirect_url) || ($script_path =~ /$.*BackgroundRebuilder\.f*cgi.*$/ ) ) {
            return 1;
        } else {
            my $file = $args{'file'};
            my $bg_rebuild = $plugin->get_config_value('BackgroundRebuilder');
            if ($bg_rebuild) {
                my $bg_rebuilder = RebuildAt1stView::BackgroundRebuild->get_by_key( { file_path => $file,
                                                                                      blog_id   => $blog_id } );
                $bg_rebuilder->save or die $bg_rebuilder->errstr;
            }
            if (-f $file) {
                unlink $file;
            }
            return 0;
        }
    }
    return 1;
}

####################################### CMS Transformer ######################################

sub _add_rebuild_status {
    my ($cb, $app, $param, $tmpl) = @_;
    $param->{'rebuilding_task'} = $plugin->get_config_value('RebuildAt1stViewTask');
}

sub _rebuild_entry {
    my ($eh, $app, $tmpl, $param) = @_;
    my $bg_rebuild = $plugin->get_config_value('BackgroundRebuilder');
    if ( $bg_rebuild ) {
        my $q = $app->param;
        my $blog_id = $q->param('blog_id');
        my $entry_id = $q->param('id');
        my $saved_changes = $q->param('saved_changes');
        if ( $saved_changes ) {
            my $entry_id = $q->param('id');
            my $app_path = $app->base.$app->path;
            $app_path .= 'plugins/RebuildAt1stView/BackgroundRebuilder.cgi?__mode=entry&blog_id=' . $blog_id . '&entry_id=' . $entry_id;
            my $old = '<mt:setvarblock name="html_head" append="1">';
            my $q_old = quotemeta($old);
            my $new = '<script src="' . $app_path . '"></script>';
            $$tmpl =~ s/$q_old/$old$new/s;
        }
    }
}

sub _finish_and_rebuild {
    my ($eh, $app, $tmpl, $param) = @_;
    my $bg_rebuild = $plugin->get_config_value('BackgroundRebuilder');
    if ( $bg_rebuild ) {
        my $q = $app->param;
        my $blog_id = $q->param('blog_id');
        my $app_path = $app->base.$app->path;
        $app_path .= 'plugins/RebuildAt1stView/BackgroundRebuilder.cgi?blog_id=' . $blog_id;
        my $old = '<mt:include name="include/chromeless_footer.tmpl">';
        my $q_old = quotemeta($old);
        my $new = '<script src="' . $app_path . '"></script>';
        $$tmpl =~ s/$q_old/$new$old/s;
    }
}

sub _rebuild_flag {
    my ($cb, $app, $param, $tmpl) = @_;
    my $q = $app->param;
    my $blog_id = $app->blog->id;
    my $get_from = 'blog:'.$blog_id;
    my $plugin_active = $plugin->get_config_value('RebuildAt1stView', $get_from);
    if ($plugin_active) {
        my $active;
        my $id = $q->param('id');
        if ($id) {
            my $template = MT::Template->load({ id => $id });
            my $type = $template->type;
            if ( ( $type =~ m/^(individual|page|category|archive)$/ ) ||  ( $type eq 'index' ) ) {
                unless ($template->build_dynamic) {
                    $active = 1;
                }
            }
        } else {
            my $type = $q->param('type');
            if ( ( $type =~ m/^(individual|page|category|archive)$/ ) ||  ( $type eq 'index' ) ) {
                $active = 1;
            }
        }
        if ($active) {
            my $template = MT::Template->load({ id => $id });
            my $pointer_field = $tmpl->getElementById('linked_file');
            my $caption = $plugin->translate('Rebuild this template at realtime');
            my $nodeset = $tmpl->createElement('app:setting', { id => 'rebuild_at', label => $plugin->translate('Plugin Options') , required => 0 });
            my $innerHTML = '<input type="checkbox" id="rebuild_at" name="rebuild_at" value="1" <mt:if name="rebuild_at">checked="checked"';
            $innerHTML .= ' </mt:if> mt:watch-change="1" /><input type="hidden" name="rebuild_at" value="0" /> <label for="rebuild_at">'.$caption.'</label>';
            $nodeset->innerHTML($innerHTML);
            $tmpl->insertAfter($nodeset, $pointer_field);
            $param->{rebuild_at} = $template->rebuild_at;
        }
    }
}

sub _RebuildAt1stViewInclude {
    my ($cb, $app, $tmpl) = @_;
    my $incl = quotemeta('<$mt:include name="include/blog_table.tmpl"$>');
    $$tmpl =~ s/$incl/&_include_blogs/e;
    my $head = quotemeta('<mt:setvarblock name="html_head" append="1">');
    my $css = <<CSS;
    <mt:setvarblock name="html_head" append="1">
    <style type="text/css">
        .RebuildAt1stView, #bt-Ra1v
                          { padding-top:5px !important;
                            text-align:center !important; }
    </style>
CSS
    $$tmpl =~ s/$head/$css/;
}

sub _RebuildAt1stViewOption {
    my ($cb, $app, $tmpl) = @_;
    my $static_url = $app->static_path;
    $$tmpl =~ s/(<td\sclass="RebuildAt1stView">)([0-9]{1,}?)(<\/td>)/$1.&_rbActive($2,$static_url).$3/eg;
}

sub _rbActive {
    my ( $blog_id, $static_url ) = @_;
    my $get_from = 'blog:'.$blog_id;
    my $plugin_active = $plugin->get_config_value('RebuildAt1stView', $get_from);
    if ( $plugin_active ) {
        return '<img src="' . $static_url . 'plugins/RebuildAt1stView/images/gpp.gif" alt="RebuildAt1stView" />';
    } else {
        return '';
    }
}

sub _RebuildAt1stViewIncludeTmplates {
    my ($cb, $app, $tmpl) = @_;
    my $q = $app->param;
    my $filter_key = $q->param('filter_key');
    if ( ( $filter_key eq 'module_templates') ||
         ( $filter_key eq 'system_templates') ||
         ( $filter_key eq 'widget_templates') ) {
         return;
    }
    my $incl = quotemeta('<$mt:include name="include/template_table.tmpl"$>');
    $$tmpl =~ s/$incl/&_include_templates/e;
    my $head = quotemeta('<mt:setvarblock name="html_head" append="1">');
    my $css = <<CSS;
    <mt:setvarblock name="html_head" append="1">
    <style type="text/css">
        .RebuildAt1stView { padding-top:5px !important;
                            text-align:center !important; }
    </style>
CSS
    $$tmpl =~ s/$head/$css/;
}

sub _RebuildAt1stViewIncludeOptionTmplates {
    my ($cb, $app, $tmpl) = @_;
    my $q = $app->param;
    my $filter_key = $q->param('filter_key');
    if ( ( $filter_key eq 'module_templates') ||
         ( $filter_key eq 'system_templates') ||
         ( $filter_key eq 'widget_templates') ) {
         return;
    }
    my $static_url = $app->static_path;
    $$tmpl =~ s/(<td\sclass="RebuildAt1stView">)([0-9]{1,}?)(<\/td>)/$1.&_rbActiveTemplate($2,$static_url).$3/eg;
}

sub _rbActiveTemplate {
    my ( $template_id, $static_url ) = @_;
    my $tmplate = MT::Template->load( { id => $template_id } );
    if ( $tmplate->rebuild_at ) {
        return '<img src="' . $static_url . 'images/status_icons/neutral.gif" alt="" />';
    } else {
        return '<img src="' . $static_url . 'images/status_icons/success.gif" alt="RebuildAt1stView" />';
    }
}

sub _include_blogs {
    return <<'TMPL';
<mt:setvarblock name="action_buttons">
    <mt:if name="is_administrator">
    <a href="javascript:void(0)"
        onclick="doRemoveItems(getByID('<$mt:var name="object_type"$>-listing-form'), '<__trans phrase="blog" escape="js">', '<__trans phrase="blogs" escape="js">'); return false;"
        accesskey="x"
        title="<__trans phrase="Delete selected blogs (x)">"
        ><__trans phrase="Delete"></a>
    </mt:if>
</mt:setvarblock>
<mtapp:listing>
    <mt:if name="__first__">
    <thead>
        <tr>
            <th class="cb"><input type="checkbox" name="id-head" value="all" class="select" /></th>
            <th id="bt-name"><__trans phrase="Blog Name"></th>
            <th id="bt-entries"><__trans phrase="Entries"></th>
            <th id="bt-comments"><__trans phrase="Comments"></th>
            <th id="bt-trackbacks"><__trans phrase="TrackBacks"></th>
            <th id="bt-authors"><__trans phrase="Users"></th>
            <th id="bt-settings"><__trans phrase="Settings"></th>
            <th id="bt-Ra1v"><img src="<mt:var name="static_uri">plugins/RebuildAt1stView/images/gpp.gif" alt="RebuildAt1stView" title="RebuildAt1stView" /></th>
        <mt:unless name="mode_search_replace">
            <mt:unless name="is_administrator">
            <th id="bt-delete"><__trans phrase="Delete"></th>
            </mt:unless>
        </mt:unless>
        </tr>
    </thead>
    <tbody>
    </mt:if>
        <tr class="<mt:if name="__odd__">odd<mt:else>even</mt:if>">
            <td class="cb"><input type="checkbox" name="id" value="<mt:var name="id">" class="select" /></td>
            <td><a href="?__mode=dashboard&amp;blog_id=<mt:var name="id">"><mt:var name="name" escape="html"></a></td>
            <td align="right"><mt:if name="can_edit_entries"><a href="<mt:var name="script_url">?__mode=list_entries&amp;blog_id=<mt:var name="id">"><mt:var name="num_entries"></a><mt:else><mt:var name="num_entries"></mt:if></td>
            <td align="right"><mt:if name="can_edit_entries"><a href="<mt:var name="script_url">?__mode=list_comments&amp;blog_id=<mt:var name="id">"><mt:var name="num_comments"></a><mt:else><mt:var name="num_comments"></mt:if></td>
            <td align="right"><mt:if name="can_edit_entries"><a href="<mt:var name="script_url">?__mode=list_pings&amp;blog_id=<mt:var name="id">"><mt:var name="num_pings"></a><mt:else><mt:var name="num_pings"></mt:if></td>
            <td align="right"><mt:var name="num_authors"></td>
            <td><mt:if name="can_edit_config"><a href="<mt:var name="script_url">?__mode=cfg_prefs&amp;blog_id=<mt:var name="id">"><__trans phrase="Settings"></a><mt:else>&nbsp;</mt:if></td>
            <mt:unless name="mode_search_replace"><mt:unless name="is_administrator"><mt:if name="can_administer_blog">
            <td><a href="javascript:void(0)" onclick="doRemoveBlog('<mt:var name="id">'); return false;"><img src="<mt:var name="static_uri">images/nav_icons/mini/delete.gif" width="9" height="11" alt="<__trans phrase="Delete">" /></a></td>
            <mt:else><td>&nbsp;</td></mt:if></mt:unless></mt:unless>
            <td class="RebuildAt1stView"><mt:var name="id"></td>
        </tr>
    <mt:if name="__last__">
    </tbody>
    </mt:if>
</mtapp:listing>
TMPL
}

sub _include_templates {
    return <<'TMPL';
<mt:setvarblock name="action_buttons">
<mt:unless name="template_type" eq="system">
    <a href="javascript:void(0)"
        onclick="doRemoveItems(getByID('<$mt:var name="object_type"$>-listing-form'), '<__trans phrase="template" escape="js">', '<__trans phrase="templates" escape="js">'); return false;"
        accesskey="x"
        title="<__trans phrase="Delete selected [_1] (x)" params="<$mt:var name="object_label_plural"$>">"
        ><__trans phrase="Delete"></a>
</mt:unless>
</mt:setvarblock>
<mtapp:listing hide_pager="1" class="show-all">
    <mt:if __first__>
        <thead>
            <tr>
                <th class="cb"><input type="checkbox" name="id-head" value="all" class="select" /></th>
                <th class="template-name"><__trans phrase="Template Name"></th>
                <th class="output-file"><__trans phrase="Output File"></th>
        <mt:unless name="filter">
                <th class="template-type"><__trans phrase="Type"></th>
        </mt:unless>
        <mt:unless name="blog_id">
                <th class="blog"><__trans phrase="Blog"></th>
        </mt:unless>
                <th class="linked-template si"><!-- <span><__trans phrase="Linked"></span> --><img src="<mt:var name="static_uri">images/status_icons/linked.gif" alt="<__trans phrase="Linked Template">" title="<__trans phrase="Linked Template">" width="17" height="9" /></th>
                <th class="dynamic-template si"><!-- <span><__trans phrase="Dynamic"></span> --><img src="<mt:var name="static_uri">images/status_icons/dynamic.gif" alt="<__trans phrase="Dynamic Template">" title="<__trans phrase="Dynamic Template">" width="11" height="9" /></th>
                <th class="rebuild-template si"><!-- <span><__trans phrase="Published w/Indexes"></span> --><img src="<mt:var name="static_uri">images/status_icons/rebuild.gif" alt="<__trans phrase="Published Template w/Indexes">" title="<__trans phrase="Published Template w/Indexes">" width="9" height="9" /></th>
                <th class="rebuild-template si"><span><img src="<mt:var name="static_uri">plugins/RebuildAt1stView/images/gpp.gif" alt="RebuildAt1stView" title="RebuildAt1stView" /></span></th>
                <th class="view-template si"><span><__trans phrase="View"></span></th>
            </tr>
        </thead>
        <tbody>
    </mt:if>
            <tr class="<mt:if name="__odd__">odd<mt:else>even</mt:if> template-<$mt:var name="template_type"$>">
                <td class="cb"><input type="checkbox" name="id" class="select" value="<mt:var name="id">" /></td>
                <td class="template-name"><a href="<mt:var name="script_url">?__mode=view&amp;_type=template&amp;id=<mt:var name="id">&amp;blog_id=<mt:var name="blog_id">"><mt:var name="name" escape="html"></a></td>
                <td class="output-file"><mt:if name="outfile"><mt:var name="outfile" escape="html"><mt:else><span><__trans phrase="-"></span></mt:if></td>
        <mt:unless name="filter">
                <td class="template-type"><$mt:var name="type"$></td>
        </mt:unless>
        <mt:unless name="blog_id">
                <td class="blog"><a href="<$mt:var name="script_url"$>?__mode=list&amp;_type=template&amp;blog_id=<$mt:var name="blog_id"$>"><$mt:var name="weblog_name" escape="html"$></a></td>
        </mt:unless>
                <td class="linked-template si status-<mt:if name="linked_file">linked<mt:else>unlinked</mt:if>"><!-- <span><__trans phrase="Linked"></span> --><img src="<mt:var name="static_uri">images/spacer.gif" alt="<mt:if name="linked_file"><__trans phrase="Yes"><mt:else><__trans phrase="No"></mt:if>" width="9" height="9" <mt:if name="linked_file">title="<mt:var name="linked_file" escape="html">"</mt:if> /></td>
                <td class="dynamic-template si status-<mt:if name="build_dynamic">dynamic<mt:else>static</mt:if>"><img src="<mt:var name="static_uri">images/spacer.gif" alt="<mt:if name="build_dynamic"><__trans phrase="Yes"><mt:else><__trans phrase="No"></mt:if>" width="9" height="9" /></td>
                <td class="rebuild-template si status-<mt:if name="rebuild_me">rebuild<mt:else>norebuild</mt:if>"><img src="<mt:var name="static_uri">images/spacer.gif" alt="<mt:if name="rebuild_me"><__trans phrase="Yes"><mt:else><__trans phrase="No"></mt:if>" width="9" height="9" /></td>
                <td class="RebuildAt1stView"><mt:var name="id"></td>
                <td class="view-template si status-view"><mt:if name="published_url"><a href="<mt:var name="published_url">" target="<__trans phrase="_external_link_target">" title="<__trans phrase="View Published Template">"><img src="<mt:var name="static_uri">images/spacer.gif" alt="<__trans phrase="View Published Template">" width="13" height="9" /></a><mt:else><__trans phrase="-"></mt:if></td>
            </tr>
    <mt:if __last__>
        </tbody>
    </mt:if>
</mtapp:listing>
TMPL
}

1;