package BackgroundRebuilder;

use strict;
use base qw( MT::App );

use MT;
use MT::Template;
use MT::WeblogPublisher;

@BackgroundRebuilder::ISA = qw(MT::App);

sub init_request {
	my $app = shift;
	$app->SUPER::init_request(@_);
	$app->add_methods( blog => \&blog );
	$app->add_methods( entry => \&entry );
	$app->add_methods( done => \&done );
	$app->{default_mode} = 'blog';
	$app->{requires_login} = 1;
	$app;
}

my $mt = MT->new(Config => '../../mt-config.cgi');

sub done {
    my $app = shift;
    return 'var res = "Rebuilding process is running...";';
}

sub entry {
    my $app = shift;
    my $q = $app->param;
    my $blog_id = $q->param('blog_id');
    my $entry_id = $q->param('entry_id');
    my $author = $app->user;
    if ( $blog_id ) {
        require MT::Permission;
        my $perms = MT::Permission->load( { blog_id => $blog_id,
                                            author_id => $author->id })
                                            or return 'Author has no permissions for blog.';
        $perms->can_rebuild or return 'Permission Denied.';
    } else {
        $author->is_superuser or return 'Permission Denied.';
    }
    my $return_path = $app->base . $app->uri() . '?__mode=done';
    my $return_path = 'Location: '.$return_path."\n\n";
    print $return_path;
    my $pid;
    $| = 1;
    FORK: {
        if( $pid = fork ) {
            close(STDOUT);
            wait;
        } elsif (defined $pid) {
            close(STDOUT);
            &bg_rebuild_entry( $blog_id, $entry_id );
            exit;
        } elsif ( $! =~ /No more process/) {
            sleep 2;
            redo FORK;
        } else {
            die "Can't fork: $\n";
        }
    }
}

sub bg_rebuild_entry {
    my ( $blog_id, $entry_id ) = @_;
    require MT::WeblogPublisher;
    my $pub = MT::WeblogPublisher->new;
    $pub->rebuild_entry(
        BlogID => $blog_id,
        Entry => $entry_id,
        BuildDependencies => 1,
        );
}

sub blog {
    my $app = shift;
    my $q = $app->param;
    my $blog_id = $q->param( 'blog_id' );
    my $author = $app->user;
    if ( $blog_id ) {
        require MT::Permission;
        my $perms = MT::Permission->load( { blog_id => $blog_id,
                                            author_id => $author->id })
                                            or return 'Author has no permissions for blog.';
        $perms->can_rebuild or return 'Permission Denied.';
    } else {
        $author->is_superuser or return 'Permission Denied.';
    }
    my $return_path = $app->base . $app->uri() . '?__mode=done';
    my $return_path = 'Location: '.$return_path."\n\n";
    print $return_path;
    my $pid;
    $| = 1;
    FORK: {
        if( $pid = fork ) {
            close(STDOUT);
            wait;
        } elsif (defined $pid) {
            close(STDOUT);
            &background_rebuild( $blog_id );
            exit;
        } elsif ( $! =~ /No more process/) {
            sleep 2;
            redo FORK;
        } else {
            die "Can't fork: $\n";
        }
    }
}

sub background_rebuild {
    my $blog_id = shift;
    my $app = MT->instance();
    my $plugin = MT::Plugin::RebuildAt1stView->instance;
    my $flag = 0;
    my $sleepTime = $plugin->get_config_value('SleepTime');
    my @remove_items = ();
    my $iter;
    unless ( $blog_id ) {
        $iter = RebuildAt1stView::BackgroundRebuild->load_iter( undef );
    } else {
        $iter = RebuildAt1stView::BackgroundRebuild->load_iter( { blog_id => $blog_id } );
    }
    while (my $rebuilder = $iter->()) {
        if (defined $rebuilder) {
            push ( @remove_items, $rebuilder );
            require MT::FileInfo;
            my $fi;
            my $file_path = $rebuilder->file_path;
            my @fis = MT::FileInfo->load({  file_path  => $file_path },
                                         {  sort => 'id',
                                            direction => 'ascend' });
            my $i = 1;
            if ( @fis > 1 ) {
                for my $finfo ( @fis ) {
                    if ($i == 1) {
                        $fi = $finfo;
                    } else {
                        $finfo->remove
                            or return $app->trans_error('Error removing fileinfo: [_1]', $finfo->errstr);
                    }
                }
                $i++;
            }
            unless ( defined $fi ) {
                $fi = MT::FileInfo->load({
                                            file_path => $file_path,
                                          },
                                          { limit => 1, }
                                         ); 
            }
            if ( $fi ) {
                unless (-f $file_path) {
                    my $template = MT::Template->load( { id => $fi->template_id } );
                    unless (defined $template) {
                        $fi->remove
                            or return $app->trans_error('Error removing fileinfo: [_1]', $fi->errstr);
                        next;
                    } else {
                        next if $template->build_dynamic;
                    }
                    require MT::WeblogPublisher;
                    my $pub = MT::WeblogPublisher->new;
                    my $res = $pub->rebuild_from_fileinfo($fi);
                    
                    unless (-f $file_path) {
                    return 'test';
                        my $at = $fi->archive_type;
                        my $entry;
                        if ( $fi->entry_id ) {
                            $entry = MT::Entry->load( { id => $fi->entry_id } );
                            if ( defined $entry ) {
                                if ( $entry-> status == 2 ) {
                                    $pub->rebuild_entry( {
                                        Entry  => $entry,
                                        Blog   => MT::Blog->load( { id => $entry->blog_id } ),
                                        BuildDependencies => 0,
                                    });
                                } else {
                                    next;
                                }
                            }
                        } else {
                            my ( $start, $end );
                            my $blog = MT::Blog->load( { id => $fi->blog_id } );
                            if ($fi->startdate) {
                                my $archiver = $pub->archiver( $at );
                                if ( my $range = $archiver->date_range ) {
                                    ( $start, $end ) = $range->( $fi->startdate );
                                    $entry = MT::Entry->load( { authored_on => [ $start, $end ] },
                                                              { range_incl => { authored_on => 1 }, limit => 1 } );
                                }
                            }
                            my $cat;
                            if ( $fi->category_id ) {
                                $cat = MT::Category->load( { id => $fi->category_id } );
                            }
                            my $map = MT::TemplateMap->load( { id => $fi->templatemap_id } );
                            my $file = $pub->archive_file_for( $entry, $blog, $at, $cat, $map );
                            unless ( defined($file) ) {
                                next;
                            }
                            $map->{__saved_output_file} = $file;
                            my $ctx = MT::Template::Context->new;
                            $ctx->{current_archive_type} = $at;
                            if ( $start && $end ) {
                                $ctx->{current_timestamp} = $start;
                                $ctx->{current_timestamp_end} = $end;
                            }
                            my $arch_root = ( $at eq 'Page' ) ? $blog->site_path : $blog->archive_path;
                            my %cond;
                            $pub->rebuild_file( $blog, $arch_root, $map, $at, $ctx, \%cond, 1, FileInfo => $fi );
                        }
                        unless (-f $file_path) {
                            require MT::Log;
                            my $log = MT::Log->new;
                            $log->message($plugin->translate('An error occurred while rebuild file [_1].', $fi->url));
                            $log->save
                                or die $log->errstr;
                        }
                    }
                    if ( $sleepTime ) {
                        sleep $sleepTime;
                    }
                }
            }
        }
    }
    if ( @remove_items ) {
        for my $rebuilder ( @remove_items ) {
            $rebuilder->remove
                or die $rebuilder->errstr;
        }
    }
}

1;
